﻿using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace RulerOverlay
{
    public class CustomInteractiveOverlay : TrackInteractiveOverlay
    {
        private InMemoryFeatureLayer lengthLableLayer;

        public CustomInteractiveOverlay()
            : base()
        {
            Collection<FeatureSourceColumn> columns = new Collection<FeatureSourceColumn>();
            columns.Add(new FeatureSourceColumn("length"));

            Collection<BaseShape> shapes = new Collection<BaseShape>();

            lengthLableLayer = new InMemoryFeatureLayer(columns, shapes);
            lengthLableLayer.Open();

            lengthLableLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = new TextStyle("length", new GeoFont("Arial", 10, DrawingFontStyles.Bold), new GeoSolidBrush(GeoColor.SimpleColors.Black));
            lengthLableLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.OverlappingRule = LabelOverlappingRule.AllowOverlapping;
            lengthLableLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.PointPlacement = PointPlacement.UpperCenter;
            lengthLableLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.SuppressPartialLabels = true;
            lengthLableLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
        }

        protected override void DrawCore(GeoCanvas canvas)
        {
            Collection<SimpleCandidate> labelingInAllLayers = new Collection<SimpleCandidate>();
            lengthLableLayer.Open();
            lengthLableLayer.Draw(canvas, labelingInAllLayers);

            if (TrackShapeLayer.InternalFeatures.Count > 0)
            {
                lengthLableLayer.InternalFeatures.Clear();

                foreach (var feature in TrackShapeLayer.InternalFeatures)
                {
                    Collection<Vertex> vertices = new Collection<Vertex>();

                    if (feature.GetShape() is PolygonShape)
                    {
                        foreach (var v in (feature.GetShape() as PolygonShape).OuterRing.Vertices)
                        {
                            vertices.Add(v);
                        }
                    }
                    else if (feature.GetShape() is LineBaseShape)
                    {
                        foreach (var v in (feature.GetShape() as LineShape).Vertices)
                        {
                            vertices.Add(v);
                        }
                    }

                    for (int i = 0; i < vertices.Count - 1; i++)
                    {
                        LineShape addingSegmentShape = new LineShape(new Collection<Vertex> { vertices[i], vertices[i + 1] });
                        Feature addingSegmentFeature = new Feature(addingSegmentShape);

                        addingSegmentFeature.ColumnValues["length"] = string.Format("{0:N3}Km", addingSegmentShape.GetLength(GeographyUnit.DecimalDegree, DistanceUnit.Kilometer));

                        lengthLableLayer.InternalFeatures.Add(addingSegmentFeature);
                    }
                }
            }

            base.DrawCore(canvas);
        }
    }
}